#include "stdafx.h"
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>
#include <stdarg.h>
#include "syntax.h"
#include "peak.h"


extern double get_sim_time(TokenString &);

//**************************************************************************

double PeakTrackObj::Evaluate()
   {
   if ( *tptr >= T1 && *tptr <= T2 )
     if ( *pointer > peak )  peak = *pointer;

   //   fprintf(stdlog,"time=%g value=%g peak=%g\n",*tptr,*pointer,peak);

   return peak;
   }

//*******************************************************************************

PeakTrackObj::PeakTrackObj(TokenString &Param, int i)
   {
   peak = 0.0;

   int p = Param.token_index(PEAK_TOKEN, i+1) - 1;

   Param.checkName(p, "Peak tracking variable");
   ID = Param.StrCpy(p);

   T1 = Param.get_double(p + 3);
   T2 = Param.get_double(p + 4);
   double Tmax = get_sim_time(Param);

   if (T1 < 0) T1 = 0.0;
   if (T2 > Tmax) T2 = Tmax;

   if ( T1 >= T2 ) Param.errorMessage(p + 3, 
                   makeMessage("Zero time interval length in \"%s\" object: [ %g, %g ]", PEAK_TOKEN, T1, T2) );
   if (VERBOSE)
       fprintf(stdlog,"  #  %s = max of %s on [%g,%g] \n", ID, Param[p+2], T1, T2);
   }


//*******************************************************************************
//*******************************************************************************
/*
void PeakTrackArray::copy(const PeakTrackArray& PTA)  
{ 
    peak_num = PTA.peak_num;
    array = new PeakTrackObj *[peak_num];

    for (int i = 0; i < peak_num; i++)
      array[i] = new PeakTrackObj(*PTA.array[i]); 
}    
*/
//*******************************************************************************

PeakTrackArray::PeakTrackArray(TokenString &Param)
{
  peak_num = Param.token_count(PEAK_TOKEN);

  array = new PeakTrackObj *[peak_num];

  if (peak_num && VERBOSE) 
       fprintf(stdlog,"\n### Setting up %d peak tracking variable(s):\n", peak_num);

  for (int i = 0; i < peak_num; i++)  array[i] = new PeakTrackObj(Param, i);

}

//*******************************************************************************


void PeakTrackArray::set_pointers(TokenString &Param, class VarList *VL)
{

  for (int i = 0; i < peak_num; i++ ) {
    int p = Param.token_index(PEAK_TOKEN, i+1) + 1;
    if ( ! ( array[i]->pointer = VL->ResolveID( Param[ p ], &(array[i]->tptr) ) ) )
      Param.errorMessage(p, StrCpy("Cannot track maximum of an undefined variable"));
  }
}

//*******************************************************************************
