#ifndef CALC_SIMULATION_H_included
#define CALC_SIMULATION_H_included


#define RETURN_NORMAL 0  // Return values for the ADI single-stepping routines
#define RETURN_END    1  // Return value when the end of simulation run has been reached


typedef int (*CaMethod)(FieldObj &, VectorObj &, BufferArray &, double dt, double);
typedef int (*BufMethod)(BufferObj &, VectorObj &, FieldObj &, double dt, double); 

//*******************************************************************************************

class SimulationObj : public VarList {

 protected:

  double       m_dt0, m_dtMax, m_accuracy0, m_accuracy;
  double       m_dtStretch, m_nStretch, m_maxError, m_ODEaccuracy, m_reduce;
  int          m_n0, m_nMax;
  char         m_PDEscheme[MAX_TOKEN_LENGTH];  

 public:

  double  totalSimTime;

  double  **DiffArray; // array of different diffusibility fields
  int     DiffNum;     // number of distinct diffusibility fields

  CaMethod CaStep;
  BufMethod BufStep;

  class TokenString *Params;
  class BCarrayObj  *BCArray;
  class RegionObj   *Synapse;
  class GridObj     *Grid;
  class FieldObj    *Ca;
  class BufferArray *Buffers;
  class KineticObj  *Gates;
  class PlotArray   *Plots;
  
  void initialize() { Params = 0; BCArray = 0; Synapse = 0; Grid = 0; Ca = 0; Buffers = 0; Gates = 0;
                      DiffArray = 0; DiffNum = 0; DiffArray = 0; Plots = 0; };

  SimulationObj()  { initialize(); }
  SimulationObj(TokenString &TS);
  ~SimulationObj();

  void initTortuosity();
  void killTortuosity();

  void Export(const char *filename);
  void Import(const char *filename);
  
  double *ResolveID(const char *, double **t=0);
  const char *ResolvePtr(double *ptr);

  class FieldObj *ResolveField(const char *);

  void SetTimeStep();
  virtual void Run();
  void FixedTimeStep(double T, int n);
  void Adaptive(double T);
};

//*******************************************************************************************

class ODESimulationObj : public SimulationObj {

 public:

  ODESimulationObj(TokenString &TS);
  ~ODESimulationObj() { this->SimulationObj::~SimulationObj(); }

  void Run();                

};


//*******************************************************************************************




#endif
